@@ -27,7 +27,7 @@ from __future__ import absolute_import
 
 __implements__ = [
     'local',
-    '_start_new_thread',
+    '_start_new_thread', # Gone in 3.13; now start_joinable_thread
     '_allocate_lock',
     'Lock',
     '_get_ident',
@@ -41,8 +41,12 @@ __implements__ = [
 ]
 
 
-import threading as __threading__
+import threading as __threading__ # imports os, sys, _thread, functools, time, itertools
 _DummyThread_ = __threading__._DummyThread
+_MainThread_ = __threading__._MainThread
+import os
+import sys
+
 from gevent.local import local
 from gevent.thread import start_new_thread as _start_new_thread
 from gevent.thread import allocate_lock as _allocate_lock
@@ -76,14 +80,19 @@ def _make_cleanup_id(gid):
 
 _weakref = None
 
+
 class _DummyThread(_DummyThread_):
     # We avoid calling the superclass constructor. This makes us about
-    # twice as fast (1.16 vs 0.68usec on PyPy, 29.3 vs 17.7usec on
-    # CPython 2.7), and has the important effect of avoiding
-    # allocation and then immediate deletion of _Thread__block, a
-    # lock. This is especially important on PyPy where locks go
-    # through the cpyext API and Cython, which is known to be slow and
-    # potentially buggy (e.g.,
+    # twice as fast:
+    #
+    # - 1.16 vs 0.68usec on PyPy (unknown version, older Intel mac)
+    # - 29.3 vs 17.7usec on CPython 2.7 (older intel Mac)
+    # - 0.98 vs 2.95usec on CPython 3.12.2 (newer M2 mac)
+    #
+    # It als has the important effect of avoiding allocation and then
+    # immediate deletion of _Thread__block, a lock. This is especially
+    # important on PyPy where locks go through the cpyext API and
+    # Cython, which is known to be slow and potentially buggy (e.g.,
     # https://bitbucket.org/pypy/pypy/issues/2149/memory-leak-for-python-subclass-of-cpyext#comment-22347393)
 
     # These objects are constructed quite frequently in some cases, so
@@ -117,6 +126,7 @@ class _DummyThread(_DummyThread_):
     _Thread__started = _started = __threading__.Event()
     _Thread__started.set()
     _tstate_lock = None
+    _handle = None # 3.13
 
     def __init__(self): # pylint:disable=super-init-not-called
         #_DummyThread_.__init__(self)
@@ -156,16 +166,111 @@ class _DummyThread(_DummyThread_):
     def __weakref_ref(self):
         return __import__('weakref').ref
 
-if hasattr(__threading__, 'main_thread'): # py 3.4+
-    def main_native_thread():
-        return __threading__.main_thread() # pylint:disable=no-member
-else:
-    def main_native_thread():
-        main_threads = [v for v in __threading__._active.values()
-                        if isinstance(v, __threading__._MainThread)]
-        assert len(main_threads) == 1, "Too many main threads"
+    # In Python 3.11.8+ and 3.12.2+ (yes, minor patch releases),
+    # CPython's ``threading._after_fork`` hook began swizzling the
+    # type of the _DummyThread into _MainThread if such a dummy thread
+    # was the current thread when ``os.fork()`` gets called.
+    # From CPython's perspective, that's a more-or-less fine thing to do.
+    # While _DummyThread isn't a subclass of _MainThread, they are both
+    # subclasses of Thread, and _MainThread doesn't add any new instance
+    # variables.
+    #
+    # From gevent's perspective, that's NOT good. Our _DummyThread
+    # doesn't have all the instance variables that Thread does, and so
+    # attempting to do anything with this now-fake _MainThread doesn't work.
+    # You in fact immediately get assertion errors from inside ``_after_fork``.
+    # Now, these are basically harmless ---  they're printed, and they prevent the cleanup
+    # of some globals in _threading, but that probably doesn't matter --- but
+    # people complained, and it could break some test scenarios (due to unexpected
+    # output on stderr, for example)
+    #
+    # We thought of a few options to patch around this:
+    #
+    # - Live with the performance penalty. Newer CPythons are making it
+    #   harder and harder to perform well, so if we can possibly avoid
+    #   adding our own performance regressions, that would be good.
+    #
+    # - ``after_fork`` uses ``isinstance(current, _DummyThread)``
+    #   before swizzling, so we could use a metaclass to make that
+    #   check return false. That's a fairly large compatibility risk,
+    #   both because of the use of a metaclass (what if some other
+    #   subclass of _DummyTHread is using an incompatible metaclass?)
+    #   and the change in ``isinstance`` behaviour. We could limit the latter
+    #   to a window around the fork, using ``os.register_at_fork(before, after_in_parent=)``,
+    #   but that's a lot of moving pieces requiring the use of a global or class
+    #   variable to track state.
+    #
+    # - We could copy the ivars of the current main thread into the
+    #   _DummyThread in ``register_at_fork(before=)``. That appears to
+    #   work, but also requires the use of
+    #   ``register_at_fork(after_in_parent=)`` to reverse it.
+    #
+    # - We could simply prevent swizzling the class in the first
+    #   place. In combination with
+    #   ``register_at_fork(after_in_child=)`` to establish a *real*
+    #   new _MainThread, that's a clean solution. Establishing a real
+    #   new _MainThread is something that CPython itself is prepared
+    #   to do if it can't figure out what the current thread is. The
+    #   compatibility risk of this is relatively low: swizzling
+    #   classes is frowned upon and uncommon, and we can limit it to
+    #   just preventing this specific case. And if somebody was
+    #   attempting this already with some other thread subclass, it
+    #   would (probably?) have the exact same issues, so we can be pretty
+    #   sure nobody is doing that.
+    #
+    # We're initially going with the last fix; the __class__ part is here,
+    # the ``after_in_child`` fixup we only apply if we're monkey-patching.
+    #
+    # Now, all of this is moot in 3.13, which takes a very different
+    # approach to handling this, and also changes some names. See
+    # https://github.com/python/cpython/commit/0e9c364f4ac18a2237bdbac702b96bcf8ef9cb09
 
-        return main_threads[0]
+    # Tests pass just fine in 3.8 (and presumably 3.9 and 3.10) with these fixes
+    # applied, but just in case, we only do it where we know it's necessary.
+    _NEEDS_CLASS_FORK_FIXUP = (
+        (sys.version_info[:2] == (3, 11) and sys.version_info[:3] >= (3, 11, 8))
+        or sys.version_info[:3] >= (3, 12, 2)
+    )
+
+    if _NEEDS_CLASS_FORK_FIXUP:
+        # Override with a property, as opposed to using __setattr__,
+        # to avoid adding overhead on any other attribute setting.
+        @property
+        def __class__(self):
+            return type(self)
+
+        @__class__.setter
+        def __class__(self, new_class):
+            # Even if we wanted to allow setting this, I'm not sure
+            # exactly how to do so when we have a property object handling it.
+            # Getting the descriptor from ``object.__dict__['__class__']``
+            # and using its ``__set__`` method raises a TypeError (as does
+            # the simpler ``super().__class__``).
+            #
+            # Better allow the TypeError for now as opposed to silently ignoring
+            # the assignment.
+            if new_class is not _MainThread_:
+                object.__dict__['__class__'].__set__(self, new_class)
+
+
+
+def _after_fork_in_child():
+    # We've already imported threading, which installed its "after" hook,
+    # so we're going to be called after that hook.
+    # Note that this is only installed when monkey-patching.
+    # TODO: Is there any point to checking to see if the current thread is
+    # our dummy thread before doing this?
+    active = __threading__._active
+    assert len(active) == 1
+    main = __threading__._MainThread()
+    __threading__._active[__threading__.get_ident()] = main
+    __threading__._main_thread = main
+    assert main.ident == __threading__.get_ident()
+
+
+
+def main_native_thread():
+    return __threading__.main_thread() # pylint:disable=no-member
 
 
 # XXX: Issue 18808 breaks us on Python 3.4+.
@@ -235,3 +340,6 @@ def _gevent_will_monkey_patch(native_module, items, warn): # pylint:disable=unus
         del __threading__._active[main_id]
         main_thread._ident = main_thread._Thread__ident = _get_ident()
         __threading__._active[_get_ident()] = main_thread
+
+    if _DummyThread._NEEDS_CLASS_FORK_FIXUP and hasattr(os, 'register_at_fork'):
+        os.register_at_fork(after_in_child=_after_fork_in_child)
